package Debian::debsigs::debsigsmain;

# SPDX-FileCopyrightText: 2001,2002 Progeny Linux Systems, Inc.
# Authors: John Goerzen, Branden Robinson
# SPDX-License-Identifier: GPL-2.0-or-later

use strict;
use warnings;

use version; our $VERSION = version->declare("v0.2.2");

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Debian::debsigs::debsigsmain - Perl extension for signing Debian packages

=head1 SYNOPSIS

  use Debian::debsigs::debsigsmain;

=head1 DESCRIPTION

This extension is undocumented so far.  Sorry.  Patches welcome.

=head1 AUTHOR

John Goerzen for Progeny Linux Systems, Inc.

=head1 SEE ALSO

perl(1).

=cut
