# A simple module loading test

# SPDX-FileCopyrightText: 2001,2002 Progeny Linux Systems, Inc.
# Authors: John Goerzen, Branden Robinson
# SPDX-License-Identifier: GPL-2.0-or-later

use Test::More tests => 4;

foreach (qw/debsigsmain arf forktools gpg/) {
	use_ok("Debian::debsigs::$_");
}
